/**
 * 
 */
package org.biojava.bio.structure;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**Holds the data of sites presented in PDB files
 *
 * @author Amr AL-Hossary
 */
public class PDBSite implements PDBRecord, Serializable {

	private static final long serialVersionUID = -4577047072916341237L;

	private String siteID = "";
	private List<Residue> residues = new Vector<PDBSite.Residue>();
	
	@Override
	public String toPDB() {
		StringBuffer buffer=new StringBuffer();
		toPDB(buffer);
		return buffer.toString();
	}

	@Override
	public void toPDB(StringBuffer buf) {
		if(residues==null || residues.size()<1)
			return;
		
		int seqNum=0;
		int residuesWritten=0;
		
		while (residuesWritten < residues.size()) {
			buf.append("SITE  ");
			buf.append(String.format(" %3d %3s %2d", seqNum+1, siteID, residues.size()));
			for (int i = 4*seqNum ; i<4 && residuesWritten< residues.size(); i++) {
				Residue residue = residues.get(i);
				buf.append(String.format(" %3s %1s%4d%1s", 
						residue.residueName,
						residue.residueNumber.chainId,
						residue.residueNumber.residueNumber,
						residue.residueNumber.insCode));
				residuesWritten++;
			}
			buf.append(System.getProperty("line.separator"));
			seqNum++;
		}
	}

	/**
	 * @param siteID the siteID to set
	 */
	public void setSiteID(String siteID) {
		this.siteID = siteID;
	}

	/**
	 * @return the siteID
	 */
	public String getSiteID() {
		return siteID;
	}

	/**
	 * @return the residues
	 */
	public List<Residue> getResidues() {
		return residues;
	}

	/**
	 * @param residues the residues to set
	 */
	public void setResidues(List<Residue> residues) {
		this.residues = residues;
	}

	/**Just a dummy bean to hold the residue data till I figure out how to do it in another way
	 * @author Amr AL-Hossary
	 */
	public static class Residue {
		private String residueName = "";
		private PDBResidueNumber residueNumber = new PDBResidueNumber();

		public Residue() {
		}

		public Residue(String residueName, PDBResidueNumber residueNumber) {
			super();
			this.residueName = residueName;
			this.residueNumber = residueNumber;
		}

		@Override
		public boolean equals(Object obj) {
			if (obj== null || ! (obj instanceof Residue)) {
				return false;
			}else {
				Residue residue = (Residue) obj;
				if (residueNumber!=null) {
					return residueNumber.equals(residue.getResidueNumber());
				}else {
					return residue.getResidueNumber()== null;// TODO shall we return true if both residue numbers are null ?
				}
			}
		}

		@Override
		public int hashCode() {
			return residueNumber.hashCode();
		}
		
		public String getResidueName() {
			return residueName;
		}

		public void setResidueName(String residueName) {
			this.residueName = residueName;
		}

		public PDBResidueNumber getResidueNumber() {
			return residueNumber;
		}

		public void setResidueNumber(PDBResidueNumber residueNumber) {
			this.residueNumber = residueNumber;
		}
	}
}
